<?php
/**
 * XFlexSlider class file.
 *
 * @author rootbear
 * @link http://www.yiiframework.com/
 * 
 * Ver. 2.0 Aug. 07, 2012
 *     - Upgrade FlexSlider to 2.0
 *     - fix code as per suggestion by Clem & Extj
 * 
 */

class XFlexSlider extends CWidget
{
	/**
	 * Default CSS class for the tab container
	 */
	const CSS_CLASS='flexslider';

	/**
	 * @var array the data that will be passed to the partial view rendered by each tab.
	 */
	public $viewData;
	
	/**
	 * @var array additional HTML options to be rendered in the container tag.
	 */
	public $htmlOptions;

	/**
	 * @var array additional HTML options to be rendered for the block inside <li> tag.
	 */
	public $innerHtmlOptions;

	public $slides=array();

	/**
	 * @var array flexsliderOptions for customizing FlexSlider's behavior.
	 */
	public $flexsliderOptions=array();
	
	private $_baseAssetsUrl;
	
	/**
	 * Runs the widget.
	 */
	public function run()
	{
		if(empty($this->slides))
			return;

		$htmlOptions=$this->htmlOptions;
		$htmlOptions['id']=$this->getId();
		if(!isset($htmlOptions['class']))
			$htmlOptions['class']=self::CSS_CLASS;

		$this->registerClientScript();
		
		echo CHtml::openTag('div',$htmlOptions)."\n";
		$this->renderSlides();
		echo CHtml::closeTag('div');
	}

	/**
	 * Registers the needed CSS and JavaScript.
	 */
	public function registerClientScript()
	{
		$assets_path = dirname(__FILE__) . '/FlexSlider';
		$this->_baseAssetsUrl = Yii::app()->assetManager->publish( $assets_path );
	
		$cs=Yii::app()->getClientScript();
		$cs->registerCoreScript('jquery');
		
		//script, css for flexslider
		$this->registerAsset('flexslider.css');
		$this->registerAsset('jquery.flexslider-min.js');
		//$this->registerAsset('jquery.flexslider.js');
		
		//by Clem: http://www.yiiframework.com/extension/xflexslider#c9337
		//$param = $this->prepareFlexSliderOptions();
		$param = CJavaScript::encode($this->flexsliderOptions);
		
		//trigger the slider
		$cs->registerScript(
		  'flexslider_trigger',
		  '
			$(window).load(function() {
				$(\'.flexslider\').flexslider(' . $param  . ');
			});
		  ',
		  CClientScript::POS_END
		);		
	}

	/**
	 * Renders slides.
	 */
	protected function renderSlides()
	{
		echo "<ul class=\"slides\">\n";
		foreach($this->slides as $id=>$slide)
		{
			echo "<li>\n";
			//echo '<div id="block" style="width:300px;height:250px;">';

			echo '<div id="flexslide-block-' . $id . '"'; 
			if (isset($this->innerHtmlOptions)){
				foreach($this->innerHtmlOptions as $key=>$value)
				{
					echo ' ' . $key . '="' . str_replace('"','\"',$value) . '"';
				}
			}
			echo ">\n";

			if(isset($slide['content']))
				echo $slide['content'];
			else if(isset($slide['url']))
			{
				echo '<iframe src="' . $slide['url'] . '"></iframe>';
			}
			else if(isset($slide['imgurl']))
			{
				echo '<img src="' . $slide['imgurl'] . '" />';
			}
			else if(isset($slide['view']))
			{
				if(isset($slide['data']))
				{
					if(is_array($this->viewData))
						$data=array_merge($this->viewData, $slide['data']);
					else
						$data=$slide['data'];
				}
				else
					$data=$this->viewData;
				$this->getController()->renderPartial($slide['view'], $data);
			}
			
			//add caption if there is
			if(isset($slide['caption']))
				echo "<p class=\"flex-caption\">" . $slide['caption'] . "</p>";
			
			echo "\n</div>\n";
			echo "</li>\n";
		}
		echo "</ul>\n";
		//echo '</div>';
	}

	/**
	 * Prepare FlexSlider's flexsliderOptions.
	 */
	protected function prepareFlexSliderOptions()
	{
		$param = "";
		foreach($this->flexsliderOptions as $key=>$value)
		{
			//normalize $value, since some of the value FlexSlider/JS requires to be quoted up
			//like 'vertical' or "vertical"
			if (strlen($value) >=2){
				if (substr($value,0,1) === '\'' && substr($value,-1) === '\'')
					$value = substr($value,1,strlen($value)-2);
					
				if (substr($value,0,1) === '"' && substr($value,-1) === '"')
					$value = substr($value,1,strlen($value)-2);
					
				if (!is_numeric($value) && !($value === 'true' || $value === 'false'))
					$value = "'" . $value . "'";
			}
		
			//by extj: http://www.yiiframework.com/extension/xflexslider#c9333
			if($value === false)
				$param .= $key . ":" . "false" . ",\n";
			else
				$param .= $key . ":" . $value . ",\n";
	
		}
		
		//by extj: For IE7 remove last ','
		//http://www.yiiframework.com/extension/xflexslider#c9333
		if ($param)
		{
			// For IE7 remove last ','
			$param = substr($param, 0, strlen($param)-2);
			$param = "{\n" . $param . "}";
		}
			
		return $param;
	}	
	
	/**
	 * generic function to register css or js
	 */
	protected function registerAsset($file)
	{
		$asset_path = $this->_baseAssetsUrl . '/' . $file;
		if(strpos($file, '.js') !== false)
			return Yii::app()->clientScript->registerScriptFile($asset_path);
		else if(strpos($file, '.css') !== false)
			return Yii::app()->clientScript->registerCssFile($asset_path);

		return $asset_path;
	}	
}
